# flake8: noqa
from vigorish.cli.menus.admin_tasks_menu import AdminTasksMenu
from vigorish.cli.menus.all_jobs_menu import AllJobsMenu
from vigorish.cli.menus.config_settings_menu import ConfigSettingsMenu
from vigorish.cli.menus.env_var_settings_menu import EnvVarSettingsMenu
from vigorish.cli.menus.jobs_menu import JobsMenu
from vigorish.cli.menus.scraped_data_errors_menu import ScrapedDataErrorsMenu
from vigorish.cli.menus.settings_menu import SettingsMenu
from vigorish.cli.menus.view_game_data_menu import ViewGameDataMenu
