# flake8: noqa
from vigorish.tasks.add_to_database import AddToDatabaseTask
from vigorish.tasks.backup_database import BackupDatabaseTask
from vigorish.tasks.calculate_avg_pitch_times import CalculateAvgPitchTimesTask
from vigorish.tasks.combine_scraped_data import CombineScrapedDataTask
from vigorish.tasks.import_scraped_data import ImportScrapedDataTask
from vigorish.tasks.patch_all_invalid_pfx import PatchAllInvalidPitchFxTask
from vigorish.tasks.restore_database import RestoreDatabaseTask
from vigorish.tasks.sync_data_no_prompts import SyncDataNoPromptsTask
from vigorish.tasks.update_player_maps import UpdatePlayerIdMapTask, UpdatePlayerTeamMapTask
